@@ -1,9 +1,24 @@
+import io
 import unittest
 import StringIO
 import cStringIO
 import copy_reg
 import sys
 
+try:
+    from unittest import skipIf
+except ImportError:
+    def skipIf(condition, message):
+        def _decorator(fn):
+            if condition:
+                return fn
+            else:
+                def skipped(self):
+                    pass
+                skipped.__doc__ = '%s skipped: %s' % (fn.__name__, message)
+                return skipped
+        return _decorator
+
 from test.test_support import TestFailed, have_unicode, TESTFN
 try:
     from test.test_support import _2G, _1M, precisionbigmemtest
@@ -13,8 +28,17 @@ except ImportError:
     def precisionbigmemtest(*args, **kwargs):
         return lambda self: None
 
+from . import _is_jython
+from . import _is_pypy
+from . import _is_pure
 from zodbpickle import pickle_2 as pickle
-from zodbpickle import _pickle as cPickle
+try:
+    from zodbpickle import _pickle as cPickle
+    has_c_implementation = not _is_pypy and not _is_pure
+except ImportError:
+    cPickle = pickle
+    has_c_implementation = False
+
 from zodbpickle import pickletools_2 as pickletools
 
 # Tests that try a number of pickle protocols should have a
@@ -1012,6 +1036,9 @@ class AbstractPickleTests(unittest.TestCase):
                              "Failed protocol %d: %r != %r"
                              % (proto, obj, loaded))
 
+    @skipIf(_is_jython, "Jython interns strings at the Java level "
+                        "but creates new PyString wrappers when __dict__ is "
+                        "accessed. See PyStringMap.")
     def test_attribute_name_interning(self):
         # Test that attribute names of pickled objects are interned when
         # unpickling.
@@ -1077,8 +1104,12 @@ class REX_five(object):
     _reduce_called = 0
     def __reduce__(self):
         self._reduce_called = 1
+        if _is_jython:
+            return super(REX_five,self).__reduce__()
         return object.__reduce__(self)
-    # This one used to fail with infinite recursion
+    # This one used to fail with infinite recursion;
+    # on Jython 2.7rc2 it still does if super() is not used; this
+    # is a bug in Jython http://bugs.jython.org/issue2323
 
 # Test classes for newobj
 
@@ -1169,6 +1200,7 @@ class AbstractPickleModuleTests(unittest.TestCase):
         s = StringIO.StringIO("X''.")
         self.assertRaises(EOFError, self.module.load, s)
 
+    @skipIf(_is_pypy or _is_jython, "Fails to access the redefined builtins")
     def test_restricted(self):
         # issue7128: cPickle failed in restricted mode
         builtins = {'pickleme': self.module,
@@ -1223,6 +1255,17 @@ class AbstractPersistentPicklerTests(unittest.TestCase):
         self.assertEqual(self.id_count, 5)
         self.assertEqual(self.load_count, 5)
 
+
+REDUCE_A = 'reduce_A'
+
+class AAA(object):
+    def __reduce__(self):
+        return str, (REDUCE_A,)
+
+class BBB(object):
+    pass
+
+
 class AbstractPicklerUnpicklerObjectTests(unittest.TestCase):
 
     pickler_class = None
@@ -1336,6 +1379,124 @@ class AbstractPicklerUnpicklerObjectTests(unittest.TestCase):
         f.seek(0)
         self.assertEqual(unpickler.load(), data2)
 
+    def test_noload_object(self):
+        global _NOLOAD_OBJECT
+        after = {}
+        _NOLOAD_OBJECT = object()
+        aaa = AAA()
+        bbb = BBB()
+        ccc = 1
+        ddd = 1.0
+        eee = ('eee', 1)
+        fff = ['fff']
+        ggg = {'ggg': 0}
+        unpickler = self.unpickler_class
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(_NOLOAD_OBJECT)
+        after['_NOLOAD_OBJECT'] = f.tell()
+        pickler.dump(aaa)
+        after['aaa'] = f.tell()
+        pickler.dump(bbb)
+        after['bbb'] = f.tell()
+        pickler.dump(ccc)
+        after['ccc'] = f.tell()
+        pickler.dump(ddd)
+        after['ddd'] = f.tell()
+        pickler.dump(eee)
+        after['eee'] = f.tell()
+        pickler.dump(fff)
+        after['fff'] = f.tell()
+        pickler.dump(ggg)
+        after['ggg'] = f.tell()
+
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        unpickler.noload() # read past _NOLOAD_OBJECT
+
+        self.assertEqual(f.tell(), after['_NOLOAD_OBJECT'])
+        noload = unpickler.noload() # read past aaa
+        self.assertEqual(noload, None)
+        self.assertEqual(f.tell(), after['aaa'])
+
+        unpickler.noload() # read past bbb
+        self.assertEqual(f.tell(), after['bbb'])
+
+        noload = unpickler.noload() # read past ccc
+        self.assertEqual(noload, ccc)
+        self.assertEqual(f.tell(), after['ccc'])
+
+        noload = unpickler.noload() # read past ddd
+        self.assertEqual(noload, ddd)
+        self.assertEqual(f.tell(), after['ddd'])
+
+        noload = unpickler.noload() # read past eee
+        self.assertEqual(noload, eee)
+        self.assertEqual(f.tell(), after['eee'])
+
+        noload = unpickler.noload() # read past fff
+        self.assertEqual(noload, fff)
+        self.assertEqual(f.tell(), after['fff'])
+
+        noload = unpickler.noload() # read past ggg
+        self.assertEqual(noload, ggg)
+        self.assertEqual(f.tell(), after['ggg'])
+
+    def test_functional_noload_dict_subclass(self):
+        """noload() doesn't break or produce any output given a dict subclass"""
+        # See http://bugs.python.org/issue1101399
+        o = MyDict()
+        o['x'] = 1
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, None)
+
+
+    def test_functional_noload_list_subclass(self):
+        """noload() doesn't break or produce any output given a list subclass"""
+        # See http://bugs.python.org/issue1101399
+        o = MyList()
+        o.append(1)
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, None)
+
+    def test_functional_noload_dict(self):
+        """noload() implements the Python 2.6 behaviour and fills in dicts"""
+        # See http://bugs.python.org/issue1101399
+        o = dict()
+        o['x'] = 1
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, o)
+
+
+    def test_functional_noload_list(self):
+        """noload() implements the Python 2.6 behaviour and fills in lists"""
+        # See http://bugs.python.org/issue1101399
+        o = list()
+        o.append(1)
+        f = io.BytesIO()
+        pickler = self.pickler_class(f, protocol=2)
+        pickler.dump(o)
+        f.seek(0)
+        unpickler = self.unpickler_class(f)
+        noload = unpickler.noload()
+        self.assertEqual(noload, o)
+
+
 class BigmemPickleTests(unittest.TestCase):
 
     # Memory requirements: 1 byte per character for input strings, 1 byte
