@@ -1,14 +1,16 @@
 import io
+import os
 import unittest
 import sys
 import copyreg
 import weakref
+import tempfile
 from http.cookies import SimpleCookie
 from zodbpickle import pickle_3 as pickle
 from zodbpickle import pickletools_3 as pickletools
 
 from test.support import (
-    TestFailed, TESTFN, run_with_locale,
+    TestFailed, run_with_locale,
     _2G, _4G, bigmemtest,
     )
 
@@ -41,6 +43,8 @@ protocols = range(pickle.HIGHEST_PROTOCOL + 1)
 
 ascii_char_size = 1
 
+fd, TESTFN = tempfile.mkstemp('.pickletester_3')
+os.close(fd)
 
 # Return True if opcode code appears in the pickle, else False.
 def opcode_in_pickle(code, pickle):
@@ -1272,6 +1276,29 @@ class AbstractPickleTests(unittest.TestCase):
             else:
                 self._check_pickling_with_opcode(obj, pickle.SETITEMS, proto)
 
+    def test_corrupted_pickle(self):
+        # Former C implementation produced corrupted pickles on these samples.
+        # See https://github.com/zopefoundation/zodbpickle/pull/47
+        sample1 = ['a'] * 17509
+        dumped = self.dumps(sample1, 0)
+        loaded = self.loads(dumped)
+        self.assertEqual(loaded, sample1)
+
+        sample2 = ['a'] * 34992
+        dumped = self.dumps(sample2, 1)
+        loaded = self.loads(dumped)
+        self.assertEqual(loaded, sample2)
+
+        sample3 = ['a'] * 34991
+        dumped = self.dumps(sample3, 2)
+        loaded = self.loads(dumped)
+        self.assertEqual(loaded, sample3)
+
+        sample4 = ['a'] * 34991
+        dumped = self.dumps(sample4, 3)
+        loaded = self.loads(dumped)
+        self.assertEqual(loaded, sample4)
+
 
 class AbstractBytestrTests(unittest.TestCase):
     def unpickleEqual(self, data, unpickled):
